@@ -77,7 +77,7 @@ void printFP(SmcVal_t val, int n, float m) {
 void printUInt(SmcVal_t val) {
   char* bytes = (char *)val.bytes;
   uint64_t data = 0;
-  for (int i = 0; i < val.dataSize; i++) {
+  for (uint32_t i = 0; i < val.dataSize; i++) {
     data += uint8_t(bytes[i]) * std::pow(256, val.dataSize - 1 -i);
   }
   std::cout << data;
@@ -112,10 +112,10 @@ void printPWM(SmcVal_t val) {
 
 void printBytesHex(SmcVal_t val) {
   std::cout << " (bytes:";
-  for (int i = 0; i < val.dataSize; i++) {
+  for (uint32_t i = 0; i < val.dataSize; i++) {
     std::ios_base::fmtflags f(std::cout.flags());
-    std::cout << " " << std::setw(2) 
-      << std::uppercase<< std::hex << std::setfill('0') 
+    std::cout << " " << std::setw(2)
+      << std::uppercase<< std::hex << std::setfill('0')
       << static_cast<unsigned int>(val.bytes[i]);
     std::cout.flags(f);
   }
@@ -171,7 +171,7 @@ kern_return_t SmcAccessor::Open() {
     std::cerr.flags(ef);
     return result;
   }
-    
+
   result = IOServiceOpen(device, mach_task_self(), 0, &conn_);
   IOObjectRelease(device);
   if (result != kIOReturnSuccess) {
@@ -193,7 +193,7 @@ kern_return_t SmcAccessor::Call(int index, SmcKeyData_t *inputStructure, SmcKeyD
   size_t   structureOutputSize;
   structureInputSize = sizeof(SmcKeyData_t);
   structureOutputSize = sizeof(SmcKeyData_t);
-    
+
   return IOConnectCallStructMethod(conn_, index, inputStructure, structureInputSize, outputStructure, &structureOutputSize);
 }
 kern_return_t SmcCall2(int index, SmcKeyData_t *inputStructure, SmcKeyData_t *outputStructure,io_connect_t conn) {
@@ -201,7 +201,7 @@ kern_return_t SmcCall2(int index, SmcKeyData_t *inputStructure, SmcKeyData_t *ou
   size_t   structureOutputSize;
   structureInputSize = sizeof(SmcKeyData_t);
   structureOutputSize = sizeof(SmcKeyData_t);
-    
+
   return IOConnectCallStructMethod(conn, index, inputStructure, structureInputSize, outputStructure, &structureOutputSize);
 }
 
@@ -210,7 +210,7 @@ kern_return_t SmcAccessor::GetKeyInfo(const uint32_t key, SmcKeyData_keyInfo_t&
   SmcKeyData_t inputStructure;
   SmcKeyData_t outputStructure;
   kern_return_t result = kIOReturnSuccess;
-    
+
   OSSpinLockLock(&g_keyInfoSpinLock);
   int i = 0;
   for (i = 0; i < g_keyInfoCacheCount; ++i) {
@@ -219,15 +219,15 @@ kern_return_t SmcAccessor::GetKeyInfo(const uint32_t key, SmcKeyData_keyInfo_t&
       break;
     }
   }
-    
+
   if (i == g_keyInfoCacheCount) {
     // Not in cache, must look it up.
     memset(&inputStructure, 0, sizeof(inputStructure));
     memset(&outputStructure, 0, sizeof(outputStructure));
-        
+
     inputStructure.key = key;
     inputStructure.data8 = kSmcCmdReadKeyInfo;
-        
+
     result = Call(kKernelIndexSmc, &inputStructure, &outputStructure);
     if (result == kIOReturnSuccess) {
       key_info = outputStructure.keyInfo;
@@ -238,9 +238,9 @@ kern_return_t SmcAccessor::GetKeyInfo(const uint32_t key, SmcKeyData_keyInfo_t&
       }
     }
   }
-    
+
   OSSpinLockUnlock(&g_keyInfoSpinLock);
-    
+
   return result;
 }
 
@@ -255,7 +255,7 @@ double SmcAccessor::ReadValue(const UInt32Char_t key) {
       std::string(val.dataType) == kDataTypeUi64) {
     char* bytes = (char *)val.bytes;
     uint64_t tmp = 0;
-    for (int i = 0; i < val.dataSize; i++) {
+    for (uint32_t i = 0; i < val.dataSize; i++) {
       tmp += uint8_t(bytes[i]) * std::pow(256, val.dataSize - 1 -i);
     }
     v = tmp;
@@ -317,37 +317,37 @@ kern_return_t SmcAccessor::ReadSmcVal(const UInt32Char_t key, SmcVal_t& val) {
   kern_return_t result;
   SmcKeyData_t  inputStructure;
   SmcKeyData_t  outputStructure;
-    
+
   memset(&inputStructure, 0, sizeof(SmcKeyData_t));
   memset(&outputStructure, 0, sizeof(SmcKeyData_t));
   memset(&val, 0, sizeof(SmcVal_t));
 
   inputStructure.key = string_util::strtoul(key, 4, 16);
-  sprintf(val.key, key);
-    
+  snprintf(val.key, sizeof(val.key), key);
+
   result = GetKeyInfo(inputStructure.key, outputStructure.keyInfo);
   if (result != kIOReturnSuccess) {
     return result;
   }
-    
+
   val.dataSize = outputStructure.keyInfo.dataSize;
-  string_util::ultostr(val.dataType, outputStructure.keyInfo.dataType);
+  string_util::ultostr(val.dataType, 5, outputStructure.keyInfo.dataType);
   inputStructure.keyInfo.dataSize = val.dataSize;
   inputStructure.data8 = kSmcCmdReadBytes;
-  
+
   result = SmcCall2(kKernelIndexSmc, &inputStructure, &outputStructure, conn_);
   if (result != kIOReturnSuccess) {
     return result;
   }
-    
+
   memcpy(val.bytes, outputStructure.bytes, sizeof(outputStructure.bytes));
-    
+
   return kIOReturnSuccess;
 }
 
 uint32_t SmcAccessor::ReadIndexCount() {
   SmcVal_t val;
-    
+
   ReadSmcVal("#KEY", val);
   return string_util::strtoul((const char *)val.bytes, val.dataSize, 10);
 }
@@ -356,29 +356,29 @@ kern_return_t SmcAccessor::PrintAll() {
   kern_return_t result;
   SmcKeyData_t  inputStructure;
   SmcKeyData_t  outputStructure;
-    
+
   int           totalKeys, i;
   UInt32Char_t  key;
   SmcVal_t      val;
-    
+
   totalKeys = ReadIndexCount();
   for (i = 0; i < totalKeys; i++) {
     memset(&inputStructure, 0, sizeof(SmcKeyData_t));
     memset(&outputStructure, 0, sizeof(SmcKeyData_t));
     memset(&val, 0, sizeof(SmcVal_t));
-    
+
     inputStructure.data8 = kSmcCmdReadIndex;
     inputStructure.data32 = i;
-    
+
     result = Call(kKernelIndexSmc, &inputStructure, &outputStructure);
     if (result != kIOReturnSuccess)
         continue;
-    
-    string_util::ultostr(key, outputStructure.key);
+
+    string_util::ultostr(key, 5, outputStructure.key);
     ReadSmcVal(key, val);
     PrintSmcVal(val);
   }
-    
+
   return kIOReturnSuccess;
 }
 
@@ -409,19 +409,19 @@ double SmcTemp::GetCpuTemp() {
   const std::pair<unsigned int, unsigned int> valid_temperature_limits{0, 110};
   // The reason why I prefer CPU die temperature to CPU proximity temperature:
   // https://github.com/narugit/smctemp/issues/2
-  temp = smc_accessor_.ReadValue(kSensorTc0d);
+  temp = smc_accessor_.ReadValue(kSensorTC0D);
   if (IsValidTemperature(temp, valid_temperature_limits)) {
     return temp;
   }
-  temp = smc_accessor_.ReadValue(kSensorTc0e);
+  temp = smc_accessor_.ReadValue(kSensorTC0E);
   if (IsValidTemperature(temp, valid_temperature_limits)) {
     return temp;
   }
-  temp = smc_accessor_.ReadValue(kSensorTc0f);
+  temp = smc_accessor_.ReadValue(kSensorTC0F);
   if (IsValidTemperature(temp, valid_temperature_limits)) {
     return temp;
   }
-  temp = smc_accessor_.ReadValue(kSensorTc0p);
+  temp = smc_accessor_.ReadValue(kSensorTC0P);
   if (IsValidTemperature(temp, valid_temperature_limits)) {
     return temp;
   }
@@ -431,7 +431,7 @@ double SmcTemp::GetCpuTemp() {
   const std::pair<unsigned int, unsigned int> valid_temperature_limits{10, 120};
 
   const std::string cpumodel = getCPUModel();
-  if (cpumodel.find("m2") != std::string::npos) {  // Apple M2
+  if (cpumodel.find("m3") != std::string::npos) {  // Apple M2
     // CPU core 1
     sensors.emplace_back(static_cast<std::string>(kSensorTp01));
     // CPU core 2
@@ -443,7 +443,24 @@ double SmcTemp::GetCpuTemp() {
     // CPU core 5
     sensors.emplace_back(static_cast<std::string>(kSensorTp05));
     // CPU core 6
-    sensors.emplace_back(static_cast<std::string>(kSensorTp0d));
+    sensors.emplace_back(static_cast<std::string>(kSensorTp0D));
+    // CPU core 7
+    sensors.emplace_back(static_cast<std::string>(kSensorTp0j));
+    // CPU core 8
+    sensors.emplace_back(static_cast<std::string>(kSensorTp0r));
+  } else if (cpumodel.find("m2") != std::string::npos) {  // Apple M2
+    // CPU core 1
+    sensors.emplace_back(static_cast<std::string>(kSensorTp01));
+    // CPU core 2
+    sensors.emplace_back(static_cast<std::string>(kSensorTp09));
+    // CPU core 3
+    sensors.emplace_back(static_cast<std::string>(kSensorTp0f));
+    // CPU core 4
+    sensors.emplace_back(static_cast<std::string>(kSensorTp0n));
+    // CPU core 5
+    sensors.emplace_back(static_cast<std::string>(kSensorTp05));
+    // CPU core 6
+    sensors.emplace_back(static_cast<std::string>(kSensorTp0D));
     // CPU core 7
     sensors.emplace_back(static_cast<std::string>(kSensorTp0j));
     // CPU core 8
@@ -454,21 +471,21 @@ double SmcTemp::GetCpuTemp() {
     // CPU performance core 2 temperature
     sensors.emplace_back(static_cast<std::string>(kSensorTp05));
     // CPU performance core 3 temperature
-    sensors.emplace_back(static_cast<std::string>(kSensorTp0d));
+    sensors.emplace_back(static_cast<std::string>(kSensorTp0D));
     // CPU performance core 4 temperature
-    sensors.emplace_back(static_cast<std::string>(kSensorTp0h));
+    sensors.emplace_back(static_cast<std::string>(kSensorTp0H));
     // CPU performance core 5 temperature
-    sensors.emplace_back(static_cast<std::string>(kSensorTp0l));
+    sensors.emplace_back(static_cast<std::string>(kSensorTp0L));
     // CPU performance core 6 temperature
-    sensors.emplace_back(static_cast<std::string>(kSensorTp0p));
+    sensors.emplace_back(static_cast<std::string>(kSensorTp0P));
     // CPU performance core 7 temperature
-    sensors.emplace_back(static_cast<std::string>(kSensorTp0x));
+    sensors.emplace_back(static_cast<std::string>(kSensorTp0X));
     // CPU performance core 8 temperature
     sensors.emplace_back(static_cast<std::string>(kSensorTp0b));
     // CPU efficient core 1 temperature
     sensors.emplace_back(static_cast<std::string>(kSensorTp09));
     // CPU efficient core 2 temperature
-    sensors.emplace_back(static_cast<std::string>(kSensorTp0t));
+    sensors.emplace_back(static_cast<std::string>(kSensorTp0T));
 
     aux_sensors.emplace_back(static_cast<std::string>(kSensorTc0a));
     aux_sensors.emplace_back(static_cast<std::string>(kSensorTc0b));
@@ -484,7 +501,7 @@ double SmcTemp::GetCpuTemp() {
     return temp;
   }
 
-  temp += CalculateAverageTemperature(aux_sensors, valid_temperature_limits);
+  temp = CalculateAverageTemperature(aux_sensors, valid_temperature_limits);
 #endif
   return temp;
 }
@@ -493,11 +510,11 @@ double SmcTemp::GetGpuTemp() {
   double temp = 0.0;
 #if defined(ARCH_TYPE_X86_64)
   const std::pair<unsigned int, unsigned int> valid_temperature_limits{0, 110};
-  temp = smc_accessor_.ReadValue(kSensorTg0d);
+  temp = smc_accessor_.ReadValue(kSensorTG0D);
   if (IsValidTemperature(temp, valid_temperature_limits)) {
     return temp;
   }
-  temp = smc_accessor_.ReadValue(kSensorTpcd);
+  temp = smc_accessor_.ReadValue(kSensorTPCD);
     if (IsValidTemperature(temp, valid_temperature_limits)) {
     return temp;
   }
@@ -505,7 +522,14 @@ double SmcTemp::GetGpuTemp() {
   std::vector<std::string> sensors;
   const std::pair<unsigned int, unsigned int> valid_temperature_limits{10, 120};
   const std::string cpumodel = getCPUModel();
-  if (cpumodel.find("m2") != std::string::npos) {  // Apple M2
+  if (cpumodel.find("m3") != std::string::npos) {  // Apple M3
+    sensors.emplace_back(static_cast<std::string>(kSensorTg0D));  // GPU 1
+    sensors.emplace_back(static_cast<std::string>(kSensorTg0P));  // GPU 2
+    sensors.emplace_back(static_cast<std::string>(kSensorTg0X));  // GPU 3
+    sensors.emplace_back(static_cast<std::string>(kSensorTg0b));  // GPU 4
+    sensors.emplace_back(static_cast<std::string>(kSensorTg0j));  // GPU 5
+    sensors.emplace_back(static_cast<std::string>(kSensorTg0v));  // GPU 6
+  } else if (cpumodel.find("m2") != std::string::npos) {  // Apple M2
     // ref: https://github.com/exelban/stats/blob/6b88eb1f60a0eb5b1a7b51b54f044bf637fd785b/Modules/Sensors/values.swift#L369-L370
     sensors.emplace_back(static_cast<std::string>(kSensorTg0f));  // GPU 1
     sensors.emplace_back(static_cast<std::string>(kSensorTg0j));  // GPU 2
